unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls;

const
  AM_GOTDATA = WM_USER + $99;
  AM_ENDSTREAM = WM_USER + 98;

type
  TMainForm = class(TForm)
    RangeCombo0: TComboBox;
    Label1: TLabel;
    ReportLabel: TLabel;
    Label2: TLabel;
    RangeCombo1: TComboBox;
    ExtremeLabel: TLabel;
    MainPaint: TPaintBox;
    PollTime: TTimer;
    GoButton: TButton;
    StopButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure RangeComboChange(Sender: TObject);
    procedure GoButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
  protected
    procedure DoGo;
    procedure DoStop;
    procedure AMGotData(var Msg: TMessage); message AM_GOTDATA;
    procedure AMEndStream(var Msg: TMessage); message AM_ENDSTREAM;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIOUSB, DetectUnit;

type
  TUSBAI1616Config = packed record
    ChannelRange: array[$0..$F] of Byte;
    CalibMode: Byte;
    TrigMode: Byte;
    StartStopCh: Byte;
    Oversample: Byte;
    StartStopChEx: Byte; //added for compatibility with the AI-MUX family
  end;

var
  DeviceIndex: LongWord;
  Config: TUSBAI1616Config;
  NextChannel, NumChannels: Integer;
  ChannelData: array of Word;
  Count0000: Int64 = 0;
  CountFFFF: Int64 = 0;

procedure ADCallback(pBuf: PWord; BufSize, Flags, Context: LongWord); cdecl; forward;

procedure TMainForm.FormCreate(Sender: TObject);
var
  DetectForm: TDetectForm;
  Status, PID, ConfigBufSize: LongWord;

begin
  IsMultiThread := True;
  DeviceIndex := diOnly;

  //If you only have the one board, QueryDeviceInfo() with diOnly will succeed
  //with the right PID. If you have more than one board, or haven't plugged it
  //in, it will fail, and this code will pop up the detector form so the user
  //can pick a board or cancel.
  Status := QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
  if (Status <> ERROR_SUCCESS) or not (((PID >= $8040) and (PID <= $8044)) or ((PID >= $8140) and (PID <= $8144))) then begin
    DetectForm := TDetectForm.Create(Self);
    if DetectForm.ShowModal = mrOK then begin
      DeviceIndex := DetectForm.DeviceIndex;
      DetectForm.Free;
    end
    else begin
      DeviceIndex := diNone;
      DetectForm.Free;
      Application.Terminate;
      Exit;
    end;
  end;

  //Set the combo boxes to their starting A/D ranges, then call RangeComboChange() to
  //set all the channels to those ranges.
  RangeCombo0.ItemIndex := 1;
  RangeCombo1.ItemIndex := 5;
  RangeComboChange(nil);
  Config.CalibMode := 0; // Take actual data, not internal calibration sources.
  Config.TrigMode := $6; // Scan selected channels each external start pin
  Config.Oversample := $FF; // 256 oversample.
  ConfigBufSize := SizeOf(Config);
  ADC_SetConfig(DeviceIndex, @Config, ConfigBufSize);
end;

procedure TMainForm.RangeComboChange(Sender: TObject);
var
  I: Integer;
  RangeCode: array[0..1] of Byte;

begin
  //The ranges in the combo box are listed in order, so that the index is equal
  //to the range code.
  RangeCode[0] := RangeCombo0.ItemIndex;
  RangeCode[1] := RangeCombo1.ItemIndex;
  for I := $0 to $F do
    Config.ChannelRange[I] := RangeCode[I and 1]
  ;
  NumChannels := 16;
  Config.StartStopCH := $F0; //Select all 16 channels, from 0 to 15.

end;

procedure TMainForm.GoButtonClick(Sender: TObject);
begin
  ReportLabel.Caption := 'Acquiring...';

  DoGo;

  RangeCombo0.Enabled := False;
  RangeCombo1.Enabled := False;
  GoButton.Enabled := False;
  StopButton.Enabled := True;
end;

procedure TMainForm.DoGo;
var
  Status: LongWord;
  ConfigBufSize: LongWord;
  Hz: Double;

begin
  //Stop the counter, in case it was running.
  Hz := 0;
  CTR_StartOutputFreq(DeviceIndex, 0, @Hz);

  ConfigBufSize := SizeOf(Config);
  ADC_SetConfig(DeviceIndex, @Config, ConfigBufSize);

  SetLength(ChannelData, 16);
  NextChannel := 0;

  Status := ADC_BulkContinuousCallbackStart(DeviceIndex, 1024, 64, 0, @ADCallback);
  if Status <> ERROR_SUCCESS then begin
    ReportLabel.Caption := 'Error ' + IntToStr(Status) + ' starting acquisition.';
    Exit;
  end;
end;

procedure ADCallback(pBuf: PWord; BufSize, Flags, Context: LongWord); cdecl;
var
  iSample: Integer;
  Sample: Word;

begin
  //Note: This is called from an alternate thread context.
  //(Specifically a worker thread spawned by the driver.)

  if BufSize <> 0 then begin
    for iSample := 0 to (BufSize div 2) - 1 do begin
      Sample := pBuf^;
      Inc(pBuf);

      if Sample = $0000 then
        Inc(Count0000)
      else if Sample = $FFFF then
        Inc(CountFFFF)
      ;
      ChannelData[NextChannel] := Sample;

      Inc(NextChannel);
      if NextChannel >= NumChannels then NextChannel := 0;
    end;

    //This tells the main thread new data is ready.
    PostMessage(MainForm.Handle, AM_GOTDATA, 0, 0);
  end;
  
  //This tells the main thread we've got the last continuous callback packet.
  if (Flags and 2) <> 0 then PostMessage(MainForm.Handle, AM_ENDSTREAM, 0, 0);
end;

procedure TMainForm.AMGotData(var Msg: TMessage);
var
  iChannel, W, X: Integer;
  R: TRect;
  XF: Double;

begin
  R := MainPaint.ClientRect;
  W := R.Right;
  XF := W / $FFFF;

  for iChannel := 0 to $F do begin
    R.Top := iChannel * 16;
    R.Bottom := R.Top + 8;
    X := Round(XF * ChannelData[iChannel]);

    MainPaint.Canvas.Brush.Color := clHighlight;
    R.Left := 0;
    R.Right := X;
    MainPaint.Canvas.FillRect(R);

    MainPaint.Canvas.Brush.Color := clBtnFace;
    R.Left := X;
    R.Right := W;
    MainPaint.Canvas.FillRect(R);
  end;
  ExtremeLabel.Caption := '0000  ' + IntToStr(Count0000)
                  + #13#10'FFFF  ' + IntToStr(CountFFFF)
  ;
end;

procedure TMainForm.AMEndStream(var Msg: TMessage);
var
  R: TRect;

begin
  ReportLabel.Caption := 'Stream ended...';

  R := MainPaint.ClientRect;
  MainPaint.Canvas.Brush.Color := clBtnFace;
  MainPaint.Canvas.FillRect(R);
end;

procedure TMainForm.StopButtonClick(Sender: TObject);
begin
  DoStop;

  GoButton.Enabled := True;
  StopButton.Enabled := False;
  RangeCombo0.Enabled := True;
  RangeCombo1.Enabled := True;
end;

procedure TMainForm.DoStop;
var
  EndStatus, IOStatus: LongWord;

begin
  EndStatus := ADC_BulkContinuousEnd(DeviceIndex, @IOStatus);

  Application.ProcessMessages; //Make sure we got the "stream ended" message first.

  if (EndStatus = ERROR_SUCCESS) and (IOStatus = ERROR_SUCCESS) then
    ReportLabel.Caption := 'Acquisition complete.'
  else if EndStatus = ERROR_SUCCESS then
    ReportLabel.Caption := 'Acquisition ended with I/O error ' + IntToStr(IOStatus) + '.'
  else
    ReportLabel.Caption := 'Error ' + IntToStr(EndStatus) + ' ending acquisition.'
  ;
end;

end.
